#!/bin/bash

##########################################################################
# Copyright (c) 2017 Stefan Jakobs <projects at localside.net>
#
# Please send all comments, suggestions, bug reports,
#    etc, to projects at localside.net.
##########################################################################
# Description 
# This script will call sa-update with default rules or sought rules only.
# It will log the update process and decide whether amavisd and/or postfix
# must be reloaded or restarted.
# 
# supported Postfix Versions: > 2.0
# supported amavisd-new Versions: > 2.7.0
##########################################################################

Version="0.5.5 (11.07.2017)"
Name="update-sa.sh"
Conf="/etc/mail/spamassassin/update-sa.conf" 

# amavis user must be able to read spamassassin config files
umask 0022

function usage {
   echo "$0 [-h|-v]"
   echo "    -h ... print this help messge"
   echo "    -v ... print version number"
   echo "Every configuration option will be read from $Conf"
   exit 0
}

function version {
   echo "$0: $Version"
   exit 0
}

while getopts 'hv' OPT; do
   case $OPT in 
      h)   usage ;;
      v)   version ;;
      *)   usage ;;
   esac
done

# try to source configuration file
if [ -r "$Conf" ]; then
   source "$Conf"
fi

# set path to other scripts
SA_UBIN=${SA_UBIN:=/usr/bin/sa-update}
SA_CBIN=${SA_CBIN:=/usr/bin/sa-compile}
AMAVISD=${AMAVISD:=/usr/sbin/rcamavis}
POSTFIX=${POSTFIX:=/usr/sbin/rcpostfix}

# set configuratin files
SITECONFIG=${SITECONFIG:=/etc/mail/spamassassin}
CHANNEL_FILE=${CHANNEL_FILE:=${SITECONFIG}/update-sa-channels.conf}
CHANNEL_KEYS=${CHANNEL_KEYS:=${SITECONFIG}/update-sa-keys.conf}
RULE_LIST=${RULE_LIST:=${SITECONFIG}/update-sa-urls.conf}

# set sought configuration
SOUGHT_CHANNEL=${SOUGHT_CHANNEL:=sought.rules.yerp.org}
SOUGHT_KEY=${SOUGHT_KEY:=6C6191E3}

# disable(0)/enable(1) ruleset compiling
COMPILE=${COMPILE:=0}

# set path to log file
SA_LOG=${SA_LOG:=/srv/log/mail/mailrelays/update-sa.log}
SA_LIB=${SA_LIB:=/var/lib/spamassassin}
SA_CP_LIB=${SA_CP_LIB:=${SA_LIB}/compiled}

# set hostname
HOST=${HOST:=$(uname -n)}

test -f $SA_LOG || touch $SA_LOG 2>/dev/null
if [ ! -w $SA_LOG ]; then
   echo "Das Logfile $SA_LOG kann nicht geschrieben werden. Abbruch."
   exit 5
elif [ ! -r $CHANNEL_FILE -o ! -r $CHANNEL_KEYS ]; then
   echo "--f Fehler beim lesen von $CHANNEL_FILE und/oder $CHANNEL_KEYS" | tee -a $SA_LOG
   exit 5
fi

echo "##### $(/bin/date +%d.%m.%Y-%R) #####" >> $SA_LOG
if [[ $1 = "sought" ]]; then
   $SA_UBIN -D --channel $SOUGHT_CHANNEL --gpgkey $SOUGHT_KEY >> $SA_LOG 2>&1
else
   $SA_UBIN -D --channelfile $CHANNEL_FILE --gpgkeyfile $CHANNEL_KEYS >> $SA_LOG 2>&1
fi
STATUS=$?

if [[ $1 != "sought" ]]; then
   while read line; do
      read cffile url rest <<< "$line"
      if [ -z "$url" ]; then
         continue
      fi
      if ! TMPFILE=$(mktemp -p $SITECONFIG tmp.XXXXXXXX) ; then 
         echo "--! Temporaere Datei konnte nicht erstellt werden." | tee -a $SA_LOG
         break
      fi
      # TMPFILE muss auf .cf enden, damit SA es einliest
      mv $TMPFILE ${TMPFILE}.cf
      TMPFILE=${TMPFILE}.cf
      echo "--! Lade $cffile herunter." >> $SA_LOG
      wget -q -O "$TMPFILE" -- $url
      if diff -q "$TMPFILE" $SITECONFIG/$cffile &> /dev/null; then
         echo "--! Keine neue Version von $cffile verfügbar." >> $SA_LOG
         rm $TMPFILE
      else
         if ! spamassassin -x -L --lint >> $SA_LOG 2>&1 ; then
            rm $TMPFILE
            echo "--f lint Test ist fehlgeschlagen!" >> $SA_LOG
            echo "--f $TMPFILE entfernt!" >> $SA_LOG
            ## Test fehlgeschlagen; verhindere amavisd reload
            [ $STATUS == "0" ] && STATUS=2
         else
            mv $TMPFILE $SITECONFIG/$cffile
            chmod 644 $SITECONFIG/$cffile
            echo "--! Neue Version von $cffile installiert!" >> $SA_LOG
            ## Update erfolgreich; stelle sicher, dass amavisd reload passiert
            [ $STATUS == "1" ] && STATUS=0
         fi
      fi
  done <<< "$(grep -v "^#" $RULE_LIST 2> /dev/null)"
fi

if [ $STATUS == "2" ]; then
   echo "--f lint Test ist fehlgeschlagen!" | tee -a $SA_LOG
   echo "Fuer mehr Infos siehe: $SA_LOG auf $HOST"
   if [ $COMPILE == "1" ]; then
      rm -r $SA_LIB/*
      echo "$SA_LIB/* gelöscht!" | tee -a $SA_LOG
   fi
elif [ $STATUS == "0" ]; then
   if [ $COMPILE == "1" ]; then
      if ! $($SA_CBIN >> $SA_LOG 2> /dev/null); then
         test -d $SA_CP_LIB && rm -r $SA_CP_LIB 
         echo "--f sa-compile Fehler! $SA_CP_LIB entfernt" | tee -a $SA_LOG
         echo "Fuer mehr Infos siehe: $SA_LOG auf $HOST"
      fi
   fi
   echo "--! Neue Channel Updates installiert." >> $SA_LOG
   $AMAVISD reload >> $SA_LOG 2> /dev/null
   sleep 120
   declare -i cnt=0
   until $AMAVISD status > /dev/null; do
      cnt=$cnt+1
      if (( $cnt >= 5 )); then
         echo "--! Fehler beim amavisd restart. Bitte manuell starten !!!" | tee -a $SA_LOG
         exit 1
      fi
      $AMAVISD restart >> $SA_LOG 2> /dev/null 
      sleep 300
   done
   if (( $cnt > 0 )); then
      if ! $POSTFIX reload >> $SA_LOG ; then
         echo "--! Fehler beim Postfix reload. Bitte manuell starten !!!" | tee -a $SA_LOG 
         exit 1
      fi
   fi
elif [ $STATUS == "1" ]; then
   echo "--! Keine Updates vorhanden" >> $SA_LOG
else
   echo "--f Fehler beim herunterladen/entpacken eines Updates" >> $SA_LOG
fi

